# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""
Module for parsing information from yaml in python accessible attributes for the ModelSuite class.
"""

import related
from attr import define
from config_builder import BaseConfigClass
from mlcvzoo_base.configuration.model_config import ModelConfig


@define
class ModelTrainerConfig(BaseConfigClass):
    """
    Class for parsing general information about the model suite and also further information
    in respective hierarchy
    """

    __related_strict__ = True

    model_config: ModelConfig = related.ChildField(cls=ModelConfig)



# Copyright 2022 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""Module for handling the download of zip files to CVAT via their commandline interface"""

import logging
import os
import shutil

import yaml
from mlcvzoo_base.utils import ensure_dir, get_file_list
from mlcvzoo_base.utils.file_utils import extract_zip_data
from related import to_yaml

from mlcvzoo_util.cvat_annotation_handler.configuration import (
    CVATCLIConfig,
    CVATTaskDumpConfig,
    CVATTaskInfoConfig,
)
from mlcvzoo_util.cvat_annotation_handler.utils import run_command_in_process

logger = logging.getLogger(__name__)


class CVATDumper:
    """
    Class for handling the download of zip files to CVAT via their commandline interface
    """

    @staticmethod
    def dump_task_data(
        dump_task_config: CVATTaskDumpConfig,
        cvat_cli_config: CVATCLIConfig,
        disable_ssl_verify: bool = False,
    ) -> None:
        """
        Execute the download of a single zip file to the CVAT instance

        Args:
            dump_task_config: Configuration of the task that is to be downloaded
            cvat_cli_config: Basis configuration that is needed for the communication between
                             the commandline-interface and CVAT
            disable_ssl_verify: Whether to allow uncertified https connections

        Returns:
            None
        """

        logger.info(
            "\n==========  DUMP TASK  ==========\n" "'''\n%s'''\n",
            to_yaml(dump_task_config, yaml_package=yaml, dumper_cls=yaml.Dumper),
        )

        target_zip_path = CVATDumper.__create_target_zip_path(
            dump_task_config=dump_task_config
        )

        if CVATDumper.__pre_clean_up_and_determine_skip(
            overwrite_existing_zip=dump_task_config.overwrite_existing_zip,
            target_zip_path=target_zip_path,
        ):
            return

        # ================================
        # DOWNLOAD TASK DATA
        CVATDumper.call_cvat_dump_via_cli(
            cvat_cli_config=cvat_cli_config,
            target_zip_path=target_zip_path,
            task_info=dump_task_config.task_info,
            disable_ssl_verify=disable_ssl_verify,
        )

        zip_extract_dir = CVATDumper.__create_zip_extract_dir_path(
            target_zip_path=target_zip_path,
            zip_extract_dir=dump_task_config.zip_extract_dir,
        )

        # ================================
        # extract and move data
        if dump_task_config.extract_data:
            extract_zip_data(
                zip_path=target_zip_path,
                zip_extract_dir=zip_extract_dir,
            )

        if dump_task_config.move_extracted_data_to_dataset_dir is not None:
            CVATDumper.__extract_and_copy_annotations_to_data_dir(
                target_zip_path=target_zip_path,
                zip_extract_dir=zip_extract_dir,
                dataset_dir=dump_task_config.dataset_dir,
            )

        # ================================
        # post clean up
        if dump_task_config.clean_up_dump_data:
            CVATDumper.__post_clean_up(
                target_zip_path=target_zip_path,
                zip_extract_dir=zip_extract_dir,
            )

    @staticmethod
    def call_cvat_dump_via_cli(
        cvat_cli_config: CVATCLIConfig,
        target_zip_path: str,
        task_info: CVATTaskInfoConfig,
        disable_ssl_verify: bool = False,
    ) -> None:
        """
        Download a zip file from CVAT.

        Args:
            cvat_cli_config: Basis configuration that is needed for the communication between
                             the commandline-interface and CVAT
            target_zip_path: The zip file to upload
            task_info: Configuration for the specific task
            disable_ssl_verify: Whether to allow uncertified https connections

        Returns:
            None
        """

        ensure_dir(file_path=target_zip_path, verbose=True)
        command = CVATDumper.__create_cvat_dump_cli_command(
            base_command=cvat_cli_config.create_base_command_string(),
            target_zip_path=target_zip_path,
            task_info=task_info,
        )

        logger.info(
            "Dump annotation data: Task-ID=%s, " "zip-path=%s",
            task_info.task_ID,
            target_zip_path,
        )
        return_code = run_command_in_process(
            command=command, disable_ssl_verify=disable_ssl_verify
        )

        if return_code != 0:
            raise IOError(f"CVAT download did not succeed, return-code={return_code}")

    @staticmethod
    def __create_target_zip_path(dump_task_config: CVATTaskDumpConfig) -> str:

        if dump_task_config.target_zip_path == "":
            target_zip_path: str = os.path.join("tmp", "tmp-task.zip")
            logger.debug(
                "dump_task_config.target_zip_path is not set, therefore the default '%s' "
                "will be used to store the downloaded task:",
                target_zip_path,
            )
        else:
            target_zip_path = dump_task_config.target_zip_path

        return target_zip_path

    @staticmethod
    def __create_zip_extract_dir_path(
        target_zip_path: str, zip_extract_dir: str
    ) -> str:

        if not os.path.isdir(zip_extract_dir):
            zip_extract_dir = target_zip_path.replace(".zip", "")
            logger.debug(
                "dump_task_config.target_zip_path is not set, therefore the default '%s' "
                "will be used to store the downloaded task:",
                zip_extract_dir,
            )

        return zip_extract_dir

    @staticmethod
    def __pre_clean_up_and_determine_skip(
        overwrite_existing_zip: bool, target_zip_path: str
    ) -> bool:

        if os.path.isfile(target_zip_path):
            if overwrite_existing_zip:
                logger.info("Clean up old dump data, remove '%s'", target_zip_path)
                os.remove(target_zip_path)
            else:
                logger.info(
                    "Skip download since zip-file '%s' exists and should not be overwritten",
                    target_zip_path,
                )
                return True

        return False

    @staticmethod
    def __post_clean_up(target_zip_path: str, zip_extract_dir: str) -> None:
        if os.path.isfile(target_zip_path):
            logger.debug("(DUMP) Clean up zip file: '%s'", target_zip_path)
            os.remove(target_zip_path)

        if os.path.isdir(zip_extract_dir):
            logger.debug(
                "(DUMP) Clean up zip extraction directory: '%s'", zip_extract_dir
            )
            shutil.rmtree(zip_extract_dir)

    @staticmethod
    def __create_cvat_dump_cli_command(
        base_command: str, target_zip_path: str, task_info: CVATTaskInfoConfig
    ) -> str:

        return (
            f"{base_command} dump "
            f"{task_info.task_ID} "
            f"{target_zip_path} "
            f"--format '{task_info.annotation_format}'"
        )

    @staticmethod
    def __extract_and_copy_annotations_to_data_dir(
        target_zip_path: str,
        zip_extract_dir: str,
        dataset_dir: str,
    ) -> None:

        if not os.path.isdir(dataset_dir):
            logger.warning(
                "Path to dataset_dir='%s' does not exist! "
                "Skip the extraction and move of annotation data.",
                dataset_dir,
            )
            return

        extract_zip_data(zip_path=target_zip_path, zip_extract_dir=zip_extract_dir)

        zip_annotation_dir = os.path.join(zip_extract_dir, "Annotations")

        annotation_file_paths = get_file_list(
            input_dir=zip_extract_dir, search_subfolders=True, file_extension=".xml"
        )

        for annotation_path in annotation_file_paths:

            dataset_annotation_path = annotation_path.replace(
                zip_annotation_dir, dataset_dir
            )

            ensure_dir(file_path=dataset_annotation_path, verbose=True)

            logger.debug(
                "Copy annotation '%s' to zip '%s'",
                annotation_path,
                dataset_annotation_path,
            )

            shutil.copy(src=annotation_path, dst=dataset_annotation_path)



# Copyright 2022 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""Module for handling the upload of zip files to CVAT via their commandline interface"""

import logging
import os
import zipfile
from typing import Dict

import yaml
from mlcvzoo_base.utils import ensure_dir, get_file_list
from related import to_yaml

from mlcvzoo_util.cvat_annotation_handler.configuration import (
    CVATCLIConfig,
    CVATTaskInfoConfig,
    CVATTaskUploadConfig,
)
from mlcvzoo_util.cvat_annotation_handler.utils import run_command_in_process

logger = logging.getLogger(__name__)


class PascalVOCUploader:
    """
    Class for handling the upload of zip files to CVAT via their commandline-interface
    """

    @staticmethod
    def upload_task_data(
        upload_task_config: CVATTaskUploadConfig,
        cvat_cli_config: CVATCLIConfig,
        disable_ssl_verify: bool = False,
    ) -> None:
        """
        Execute the upload of a single zip file to the CVAT instance

        Args:
            upload_task_config: Configuration of the task that is to be uploaded
            cvat_cli_config: Basis configuration that is needed for the communication between
                             the commandline-interface and CVAT
            disable_ssl_verify: Whether to allow uncertified https connections

        Returns:
            None
        """

        logger.info(
            "\n========== UPLOAD TASK ==========\n" "Task config: \n" "'''\n%s'''\n",
            to_yaml(
                upload_task_config,
                yaml_package=yaml,
                dumper_cls=yaml.Dumper,
            ),
        )

        if upload_task_config.use_prediction_data:
            target_zip_path = PascalVOCUploader.write_upload_zip_file(
                upload_task_config=upload_task_config
            )
        else:
            target_zip_path = upload_task_config.source_zip_path
            upload_task_config.target_zip_path = target_zip_path

        if upload_task_config.execute_upload:
            PascalVOCUploader.call_cvat_upload_via_cli(
                cvat_cli_config=cvat_cli_config,
                target_zip_path=target_zip_path,
                task_info=upload_task_config.task_info,
                disable_ssl_verify=disable_ssl_verify,
            )

        if upload_task_config.clean_up_upload_data:
            PascalVOCUploader.__post_clean_up(
                source_zip_path=upload_task_config.source_zip_path,
                target_zip_path=target_zip_path,
            )

    @staticmethod
    def __post_clean_up(source_zip_path: str, target_zip_path: str) -> None:
        if os.path.isfile(source_zip_path):
            logger.info("(UPLOAD) Clean up dump zip file: '%s'", source_zip_path)
            os.remove(source_zip_path)

        if os.path.isfile(target_zip_path):
            logger.info("(UPLOAD) Clean up upload zip file: '%s'", target_zip_path)
            os.remove(target_zip_path)

    @staticmethod
    def call_cvat_upload_via_cli(
        cvat_cli_config: CVATCLIConfig,
        target_zip_path: str,
        task_info: CVATTaskInfoConfig,
        disable_ssl_verify: bool = False,
    ) -> None:
        """
        Upload a zip file to CVAT.

        Args:
            cvat_cli_config: Basis configuration that is needed for the communication between
                             the commandline-interface and CVAT
            target_zip_path: The zip file to upload
            task_info: Configuration for the specific task
            disable_ssl_verify: Whether to allow uncertified https connections

        Returns:
            None
        """

        # ================================
        # UPLOAD TASK DATA
        command = PascalVOCUploader.__create_cvat_upload_cli_command(
            base_command=cvat_cli_config.create_base_command_string(),
            target_zip_path=target_zip_path,
            task_info=task_info,
        )
        logger.info(
            "Upload annotation data: Task-ID=%s, " "zip-path=%s",
            task_info.task_ID,
            target_zip_path,
        )
        return_code = run_command_in_process(
            command=command, disable_ssl_verify=disable_ssl_verify
        )

        if return_code != 0:
            raise ValueError(f"Upload did not succeed, return-code={return_code}")

    @staticmethod
    def __create_cvat_upload_cli_command(
        base_command: str, target_zip_path: str, task_info: CVATTaskInfoConfig
    ) -> str:

        return (
            f"{base_command} upload "
            f"{task_info.task_ID} "
            f"{target_zip_path} "
            f"--format '{task_info.annotation_format}'"
        )

    @staticmethod
    def write_upload_zip_file(upload_task_config: CVATTaskUploadConfig) -> str:
        """
        Create a zip file based on generated PASCAL-VOC xml files,
        that can be uploaded to a CVAT instance.

        Args:
            upload_task_config: Configuration that states where to find the prediction data
                                and where the zip file should be saved

        Returns:
            The path to the zip file that has been written to disk
        """

        # Generate path to the zip-file which should be uploaded to CVAT
        if upload_task_config.target_zip_path == "":
            target_zip_path = upload_task_config.source_zip_path.replace(
                ".zip", "_upload.zip"
            )
        else:
            target_zip_path = upload_task_config.target_zip_path

        # CLEAN UP old upload zip-file
        if os.path.isfile(target_zip_path):
            logger.info("(UPLOAD) Clean up zip file: '%s'", target_zip_path)
            os.remove(target_zip_path)

        return PascalVOCUploader.__write_prediction_data_to_zip(
            source_zip_path=upload_task_config.source_zip_path,
            target_zip_path=target_zip_path,
            prediction_data_dir=upload_task_config.prediction_data_dir,
        )

    @staticmethod
    def __write_prediction_data_to_zip(
        source_zip_path: str, target_zip_path: str, prediction_data_dir: str
    ) -> str:

        if not os.path.isdir(prediction_data_dir):
            raise ValueError(
                f"prediction_data_dir='{prediction_data_dir}' does not exist! "
                "Please specify a directory in order to generate"
                "a prediction zip file for the upload to CVAT!"
            )

        # Open zip-file to write data which will be uploaded to the CVAT task
        ensure_dir(file_path=target_zip_path, verbose=True)
        target_zip_file = zipfile.ZipFile(  # pylint: disable=consider-using-with
            target_zip_path, "w"
        )

        # Open original zip file to read content that shouldbe copied
        source_zip_file = zipfile.ZipFile(
            source_zip_path
        )  # pylint: disable=consider-using-with
        original_zip_paths = zipfile.ZipFile.namelist(source_zip_file)
        for file_path in original_zip_paths:
            # Copy the 'labelmap.txt' of the original zip file to the one
            # that is uploaded
            if file_path == "labelmap.txt":
                target_zip_file.writestr(
                    zinfo_or_arcname="labelmap.txt",
                    data=source_zip_file.read("labelmap.txt"),
                )

            # Copy every file of the 'ImageSets' folder of the original zip file,
            # to the zip file that is uploaded
            if file_path.split("/")[0] == "ImageSets":
                target_zip_file.writestr(
                    zinfo_or_arcname=file_path, data=source_zip_file.read(file_path)
                )

        # NOTE:
        # - The annotations are defined in the download cvat zip file
        # - Therefore it is enough to specify a root directory where
        #   all xml files can be parsed from.
        # - The decision which xml files should be put in the upload
        #   zip file, is made on the basis of the information from the
        #   downloaded zip file from "ImageSets/Main/default.txt"
        predicted_xml_paths = get_file_list(
            input_dir=prediction_data_dir,
            file_extension=".xml",
            search_subfolders=True,
        )

        predicted_xml_paths_map: Dict[str, str] = {}
        for predicted_xml_path in predicted_xml_paths:
            predicted_xml_paths_map[
                os.path.basename(predicted_xml_path).replace(".xml", "")
            ] = predicted_xml_path

        # Get all xml paths that are part of the task (represented by the zip)
        annotation_xml_files = (
            source_zip_file.read("ImageSets/Main/default.txt")
            .decode("utf-8")
            .split("\n")
        )

        # Write annotation data to zip-file
        for annotation_xml_path in annotation_xml_files:
            annotation_file_name = os.path.basename(annotation_xml_path)

            if annotation_file_name in predicted_xml_paths_map:

                logger.debug(
                    "Add xml file to upload zip-file: '%s'",
                    predicted_xml_paths_map[annotation_file_name],
                )
                with open(
                    file=predicted_xml_paths_map[annotation_file_name], encoding="utf8"
                ) as prediction_file:
                    target_zip_file.writestr(
                        zinfo_or_arcname=f"Annotations/{annotation_xml_path}.xml",
                        data="".join(prediction_file.readlines()),
                    )

        source_zip_file.close()
        target_zip_file.close()

        logger.info(
            "Generated prediction zip file '%s' for upload to cvat.", target_zip_path
        )

        return target_zip_path



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mlcvzoo_util',
 'mlcvzoo_util.cvat_annotation_handler',
 'mlcvzoo_util.logger',
 'mlcvzoo_util.model_evaluator',
 'mlcvzoo_util.model_timer',
 'mlcvzoo_util.model_trainer',
 'mlcvzoo_util.pre_annotation_tool',
 'mlcvzoo_util.video_image_creator']

package_data = \
{'': ['*']}

install_requires = \
['imageio>=2.9,<3.0',
 'mlcvzoo_base>=5.0,<6.0',
 'mlflow>=1.22,<2.0',
 'nptyping>=2.0,<3.0',
 'numpy>=1.19.2,!=1.19.5',
 'opencv-contrib-python>=4.5,<5.0,!=4.5.5.64',
 'opencv-python>=4.5,<5.0,!=4.5.5.64',
 'related-mltoolbox>=1.0,<2.0',
 'tqdm>=4.61,<5.0',
 'yaml-config-builder>=8.0,<9.0']

entry_points = \
{'console_scripts': ['mlcvzoo-cvat-handler = '
                     'mlcvzoo_util.cvat_annotation_handler.cvat_annotation_handler:main',
                     'mlcvzoo-modelevaluator = '
                     'mlcvzoo_util.model_evaluator.model_evaluator:main',
                     'mlcvzoo-modeltimer = '
                     'mlcvzoo_util.model_timer.model_timer:main',
                     'mlcvzoo-modeltrainer = '
                     'mlcvzoo_util.model_trainer.model_trainer:main',
                     'mlcvzoo-preannotator = '
                     'mlcvzoo_util.pre_annotation_tool.pre_annotation_tool:main',
                     'mlcvzoo-video-image-creator = '
                     'mlcvzoo_util.video_image_creator.video_image_creator:main']}

setup_kwargs = {
    'name': 'mlcvzoo-util',
    'version': '0.3.0',
    'description': 'MLCVZoo Util Package',
    'long_description': '# MLCVZoo Util\n\nThe MLCVZoo is an SDK for simplifying the usage of various (machine learning driven)\ncomputer vision algorithms. The package **mlcvzoo_util** provides additional utility functions for downstream developments and tools for convenience and CLI usage.\n\nFurther information about the MLCVZoo can be found [here](../README.md).\n\n## Install\n`\npip install mlcvzoo-util\n`\n\n## Technology stack\n\n- Python\n',
    'author': 'Maximilian Otten',
    'author_email': 'maximilian.otten@iml.fraunhofer.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://git.openlogisticsfoundation.org/silicon-economy/base/ml-toolbox/mlcvzoo',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# Copyright 2022 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""
Module for configuring the parsing of information from yaml in python
accessible attributes for the PreAnnotationTool class
"""

import related
from config_builder import BaseConfigClass
from mlcvzoo_base.configuration.model_config import ModelConfig
from mlcvzoo_base.configuration.structs import OpenCVImageFormats

from mlcvzoo_util.cvat_annotation_handler.configuration import (
    CVATAnnotationHandlerConfig,
)


@related.mutable(strict=False)
class PreAnnotateCVATConfig(BaseConfigClass):
    """
    Main configuration class for the PreAnnotationTool
    """

    model_config: ModelConfig = related.ChildField(cls=ModelConfig)

    cvat_annotation_handler_config: CVATAnnotationHandlerConfig = related.ChildField(
        cls=CVATAnnotationHandlerConfig
    )

    root_image_dir: str = related.StringField()
    input_image_type: str = related.StringField()
    output_xml_dir: str = related.StringField()

    generate_annotations: bool = related.BooleanField()
    overwrite_existing_annotations: bool = related.BooleanField()

    show_predictions: bool = related.BooleanField(required=False, default=False)

    def check_values(self) -> bool:
        return True



# Copyright 2021 Open Logistics Foundation
#
# Licensed under the Open Logistics License 1.0.
# For details on the licensing terms, see the LICENSE file.

"""
Module for configuring the parsing of information from yaml in python
accessible attributes for extracting frames from videos
"""
from typing import Dict

import related
from config_builder import BaseConfigClass


@related.mutable(strict=True)
class VideoImageCreatorConfig(BaseConfigClass):
    """Class for parsing information for video image creator tool"""

    default_step_width_map: Dict[str, int] = {
        "1": 1,
        "2": 2,
        "3": 3,
        "4": 4,
        "5": 5,
        "6": -1,
        "7": -2,
        "8": -3,
        "9": -4,
        "0": -5,
    }

    step_width_map: Dict[str, int] = related.ChildField(
        cls=dict, default=default_step_width_map
    )

    video_file_extension: str = related.StringField(default=".mp4")

    video_input_dir: str = related.StringField(default="")

    video_input_path: str = related.StringField(default="")

    winname: str = related.StringField(default="VideoImageCreator")

    window_size: int = related.IntegerField(default=500)
    resize_window: bool = related.BooleanField(default=True)
